c
c Program : DECINC
c
c Revision : 2.0, Jan. 13, 1989
c
c Purpose : To calculate IGRF/DGRF values at positions specified
c           by elevation and geographic coordinates.
c           Tested in northern hemisphere only.
c
c Author  : John Broome / G. Haines
c           Potential Fields Section
c           Geological Survey of Canada
c           Ottawa, Canada
c
c        **  Crown Copywrite **
c
c Part of Geological Survey of Canada Open File 1334
c
*******************************************************************
c
      character cmodel*6, cfile*40, ed*4
      logical*2 fex
      data icfil/1/
      ed = char(27) // '[2J' 
c
      write(*,1111) ed
1111  format(1x,a,\)
      write(*,*)' '
      write(*,*)'       ----------------------------------'      
      write(*,*)'       |                                |'
      write(*,*)'       |  *******  DECINC 2.0  *******  |'
      write(*,*)'       |                                |'  
      write(*,*)'       ----------------------------------'
      write(*,*) ' '
      write(*,*)' This program calculates the inclination, '
      write(*,*)' declination, and magnitude of the earth''s field'
      write(*,*)' at a point identified by latitude, longitude '
      write(*,*)' and altitude.'
      call hret
c
10    write(*,1111) ed
      write(*,1050)
1050  format(/,'       1 - DGRF 1970',/,
     +        '       2 - IGRF 1975',/,
     +        '       3 - DGRF 1975',/,
     +        '       4 - DGRF 1980',/,
     +        '       5 - IGRF 1985',//,
     +        '  Choose coefficient set for field calculation: ',\)
c
      read(*,*,err=10) icoef
      if(icoef.lt.1.or.icoef.gt.5) then
        call error('Enter 1 to 5 !')
        call hret
        go to 10
      endif
c
      if(icoef.eq.1) then
        cfile = 'dgrf70.dat'
        cmodel = 'DGRF70'
        nmax = 10
      else if (icoef.eq.2) then
        cfile = 'igrf75.dat'
        cmodel = 'IGRF75'
        nmax = 8
      else if (icoef.eq.3) then
        cfile = 'dgrf75.dat'
        cmodel = 'DGRF75'
        nmax = 10
      else if (icoef.eq.4) then
        cfile = 'dgrf80.dat'
        cmodel = 'DGRF80'
        nmax = 10
      else if (icoef.eq.5) then
        cfile = 'igrf85.dat'
        cmodel = 'IGRF85'
        nmax = 10
      endif
c
      inquire(file=cfile,exist=fex)
      if(fex.eqv..false.)then
        call error('Coefficients file not found')
        call hret
        go to 10
      endif
c
      open(icfil,file=cfile,status='old',form='formatted')
c
      write(*,1111) ed
      write(*,'(/a\)')' Enter altitude of the point (km) : '
      read(*,*) alt
c
      write(*,'(/a\)')' Enter year for temporal correction: '
      read(*,*) tm
c
      l = 1
c
20    write(*,'(/a)')' Enter decimal latitude of point,'
      write(*,'(a\)')'  (-9999 to quit program)...........: '
      read(*,*) dlat
      if(dlat.eq.-9999) go to 999
      write(*,'(/a)')' Enter decimal longitude of point,'
      write(*,'(a\)')'  ( negative for West ) : '
      read(*,*) dlong
c  
c     write(*,*)dlat,dlong,alt,tm,nmax,l,bn,be,bv,f
      call field(dlat,dlong,alt,tm,nmax,l,bn,be,bv,f)
c
c Calculate declination and dip of field 
c
      dec = atan ( be/bn ) * 57.30
      bh = sqrt ( bn**2 + be**2 )
      dip = atan ( bv/bh ) * 57.30
c
      write(*,1111) ed
      write(*,*)' '
      write(*,*)'         ',cmodel
      write(*,*)'         ------'
c
      write(*,1070)dlat,dlong,alt,tm,nmax,bn,be,bv,f,dec,dip
1070  format( /,' Latitude                  : ',f8.3,/,
     +          ' Longitude (neg. for West) : ',f8.3,/,
     +          ' Altitude    (km)          : ',f8.3,/,
     +          ' Year                      : ',f5.0,/,
     +          ' Number of coefficients.    : ',i5,/,
     +          '------------------------------------------',/,
     +          ' North component    (nanoteslas) : ',f10.3,/,
     +          ' East component     (nanoteslas) : ',f10.3,/,
     +          ' Vertical component (nanoteslas) : ',f10.3,/,
     +          ' Field strength     (nanoteslas) : ',f10.3,/,
     +          ' Declination(Azimuth in degrees) : ',f10.3,/,
     +          ' Dip                   (degrees) : ',f10.3,/)
c
      call hret
      go to 20
c
999   stop
      end
c
c
******************************************************************
            SUBROUTINE FIELD(DLAT,DLONG,ALT,TM,NMAX,L,BN,BE,BV,F)
c
c Revision : 1.0, Aug 10, 1988, Broome
c
c Purpose : To calculate the geomagnetic field components from a 
c           coeficient set.
c
c Parameters :
c          Dlat  - Decimal latitude of point
c          Dlong - Decimal longitude of point
c          Alt   - Altitude of point in km
c          Tm    - Year, for temporal correction
c          Nmax  - number of coeficients  to be used for
c                       calculation
c          l     - set positive to read coefficient set from lun1
c          Bn    - output, field component in North direction
c          Be    - output, field component in East  direction
c          Bv    - output, field component in vertical direction
c          F     - output, field magnitude
c
c     (geomagnetic field components x,y,z, in gammas.)
c     (input dlat,alt in geodetic coordinates)
c     (output bn,bv also in geodetic.)
c 
c     Originally written by : G. V. Haines, 
c                             Division of Geomagnetism,
c                             Earth Physics Branch
c                             Energy Mines and Resources Canada
c                             1 Observatory Cres., Ottawa,
c                             Ontario    K1A 0Y3.
c 
c
      dimension p(21,21),g(21,21),h(21,21),dp(21,21), const(21,21)
      dimension sp(21), cp(21), shmit(21,21), ht(21,21), gt(21,21)
      dimension htt(21,21),gtt(21,21),th(21,21),tg(21,21),fm(21)
      dimension fn(21),aid(4)
      character*4 aid
      data ndim/21/
c
      if (l .Eq. 0)  Go to 15
      if (l .Lt. 0)  Go to 33
c
c Initialize arrays 
c
      do 100 n=1,21
        fm(n) = 0
        fn(n) = 0.0
        sp(n) = 0.0
        cp(n) = 0.0
        do 100 m=1, 21
          p(n,m) = 0.0
          g(n,m) = 0.0
          h(n,m) = 0.0
          dp(n,m) = 0.0
          ht(n,m) = 0.0
          gt(n,m) = 0.0
          th(n,m) = 0.0
          tg(n,m) = 0.0
          gtt(n,m) = 0.0
          htt(n,m) = 0.0
          const(n,m) = 0.0
          shmit(n,m) = 0.0
100   continue
c
c Read in coefficients from file and store in arrays
c
      read(1,1027) j,k,tzero,re,a,flat,(aid(i),i=1,4)    
1027  format (2i1,f8.2,3F10.3,4A10)
c
c        j       - zero if coefficients are for oblate earth
c                - non-zero if coefficients are for spherical
earth.
c        k       - zero if coefficients are schmidt normalized
c                - non-zero if gauss normalized.
c        tzero   - epoch year of coefficients.
c        re      - radius of equivalent spherical earth used in
c                  deriving the coefficients.
c        a       - semi-major axis of geodetic ellipsoid.
c        flat    - flattening of ellipsoid.
c        aid     - identification (maximum 40 characters)
 
c     print 1028, (aid(i),i=1,4),j,k,tzero,re,a,flat
1028  format (32h0Spherical Harmonic Coefficients 9x 4a10/
     +4x1hj 3x1hk 7x5htzero 10x2hre 11x1ha 8x4hflat/1x,2i4,f12.2,
     +3f12.3/9H    n   m 9x3hgnm 9x3hhnm 8x4hgtnm 8x4hhtnm 7x5
     +hgttnm 7x5hhttnm)
      maxn=0
      temp=0.
c
c If coefficients are for oblate earth calculate oblate
c   coefficients.
c
      If (j .eq. 0) then
        flat= 1.-1./flat
        a2=a**2
        a4=a**4
        b2=(a*flat)**2
        a2b2=a2*(1.-flat**2)
        a4b4=a4*(1.-flat**4)
      endif
c
c Read coefficients into arrays from the selected file
c
25    read(1,1002) n,m,gnm,hnm,gtnm,htnm,gttnm,httnm    
1002  format (2i4,6f12.4)
c     write(*,1006)n,m,gnm,hnm,gtnm,htnm,gttnm,httnm
1006  format (1h ,2i4,6f12.4,10X, 8hnot used)
      if (n .gt. 0)  then
        if (n .lt. ndim) then
          n=n+1
          m=m+1
          maxn=  max0 (n,maxn)
          g(n,m)=gnm
          h(n,m)=hnm
          gt(n,m)=gtnm
          ht(n,m)=htnm
          gtt(n,m)=gttnm
          htt(n,m)=httnm
          temp=amax1(temp,abs (gtnm))
        else
          write(*,*)'Warning: Number of coeficients in file'
          write(*,*)' exceeds dimension limit of ',ndim
        endif
        go to 25
      endif
c
c     write(*,*)   ((fm(n),fm(m),g(n,m),h(n,m),gt(n,m),ht(n,m),
c    +     gtt(n,m),htt(n,m),  m=1,n),n=2,maxn)
1024  format (1h ,2f4.0,6F12.4)
c
c Check if number of coeficients requested exceed number in 
c  file.
c
      if (nmax .ge. maxn) then
        maxnm1 = maxn-1
        call error('No. of coeficients requested exceed no. in file')
        stop
      endif
c
      if (temp .eq. 0) then
        l = -l
      else
        l=0
      endif
c
c Calculate Schmidt normalization coefficients
c
      p(1,1)=1.
      dp(1,1)=0.
      sp(1)=0.
      cp(1)=1.
      rad=57.295779513
      shmit(1,1)=-1.
      fm(1) = 0.
      do 20 n=2,ndim
        shmit(n,1)=shmit(n-1,1)*float (2*n-3)/float (n-1)
        fn(n)=n
        fm(n) = n - 1
        jj=2
        do 20 m=2,n
          shmit(n,m)=shmit(n,m-1)*sqrt (float ((n-m+1)*jj)
     +    /float(n+m-2))
          jj=1
20      continue
c
      do 4 n=3,ndim
        nm2 = n-2
        do 4 m=1,nm2
          const(n,m)=float ((n-2)**2-(m-1)**2)/
     +    float((2*n-3)*(2*n-5))
4     continue
c
c Schmidt normalize coeficients if not already done
c
      if (k .eq. 0) then
        do 32 n=2,maxn
          do 32 m=1,n
            g(n,m)=g(n,m)*shmit(n,m)
            h(n,m)=h(n,m) *shmit(n,m)
            gt(n,m)=gt(n,m)*shmit(n,m)
            ht(n,m)=ht(n,m)*shmit(n,m)
            gtt(n,m)=gtt(n,m)*shmit(n,m)
            htt(n,m)=htt(n,m)*shmit(n,m)
32      continue
        tlast = -99
      endif
c
c Recalculate temporal correction
c
15    if (tm .ne. tlast) then
        t=tm-tzero
        do 22 n=2,maxn
          do 22 m=1,n
          tg(n,m)=g(n,m)+t *(gt(n,m)+gtt(n,m)*t)
22      th(n,m)=h(n,m)+t *(ht(n,m)+htt(n,m)*t)
        tlast=tm
      endif
c
33    rlat=dlat/rad
      sinla=sin (rlat)
      rlong=dlong/rad
      cp(2)=cos (rlong)
      sp(2)=sin (rlong)
      nmaxp1=nmax+1
c
      do 10 m=3,nmaxp1
        sp(m)=sp(2)*cp(m-1)+cp(2)*sp(m-1)
        cp(m)=cp(2)*cp(m-1)-sp(2)*sp(m-1)
10    continue
c
      if (j .ne. 0) then
        r = re + alt
        ct=sinla
        go to 21
      endif
c
      sinla2=sinla*sinla
      den2=a2-a2b2*sinla2
      den=sqrt (den2)
      cosla = sqrt(1.-sinla2)
c
      if (cosla .eq. 0.)  then
        theta = 90./rad
        if (sinla .Lt. 0.)  theta = -theta
      else
        altden = alt*den
        theta = atan((altden+b2)/(altden+a2)*sinla/cosla)
      endif
c
      r=sqrt (alt*(alt+2.*Den)+(a4-a4b4*sinla2)/den2)
      ct=sin (theta)
c
21    stst = 1. - ct**2
      st = sqrt(stst)
      ctst = ct*st
      aor = re/r
      ar=aor*aor
      bn=0.
      be=0.
      bv=0.
c
c Calculate the 3 components of the field
c
      do 54 n=2,nmaxp1
        sumn=0.
        sume=0.
        sumv=0.
        ar=aor*ar
c
        if (n .lt. 3) then
          p(2,2) = 1.
          dp(2,2) = ct
        else
          p(n,n) = st* p(n-1,n-1)
          dp(n,n) = st*dp(n-1,n-1) +  ctst*p(n-1,n-1)
        endif
        fact = st
c
        do 45 m=1,n
          temp = tg(n,m)*cp(m) + th(n,m)*sp(m)
          if (m .eq. 2)  fact = stst
          if (m .le. n-2)  go to 12
          if (n .eq. m)  go to 13
          p(n,m) = ct* p(n-1,m)
          dp(n,m) = ct*dp(n-1,m) - fact*p(n-1,m)
          go to 13
12        p(n,m) = ct* p(n-1,m) - const(n,m)* p(n-2,m)
          dp(n,m) = ct*dp(n-1,m)-const(n,m)*dp(n-2,m)-fact*p(n-1,m)
c
13        sumn = sumn - dp(n,m)*temp
          if (m .ne. 1) then
            sume = sume+p(n,m)*fm(m)*(-tg(n,m)*sp(m)+th(n,m)*cp(m))
            temp = temp*st
          endif
          sumv = sumv + p(n,m)*temp
45      continue
        bn = bn + sumn*ar
        be = be + sume*ar
        bv = bv + sumv*ar*fn(n)
54    continue
c
c    Transforms field to geodetic directions if required
c
      if (j .eq. 0) then
        sind=sin (rlat-theta  )
        cosd=sqrt (1.0-sind*sind)
        tn=bn
        bn=bn*cosd+bv*sind
        bv=bv*cosd-tn*sind
      endif
c
      f=sqrt(bn**2+be**2+bv**2)
      return
      end
c
c
******************************************************************
      SUBROUTINE HRET
c
c Purpose: This routine tells user to hit a key to continue
c
      character*1 dum
      integer     iocheck
c
      write (*,10)
10    format (/, 1x, 'Hit ENTER to continue. ', \)
      read (*, '(a)', iostat=iocheck) dum
c
      END
c
c *****************************************************************
      SUBROUTINE ERROR (STRING)
c
c Purpose : To display bold, blinking error messages on the screen
c           and call the error sound subroutine
c
      character  string*(*), esc*1, csi*2, sgr5*4, sgr1*4
      character  sgr0*4
c
      ESC    = char(27)
      CSI    = esc // '['
      SGR5   = csi // '5m'
      SGR1   = csi // '1m'
      SGR0   = csi // '0m'
c
      write(*,1111) sgr1,sgr5
1111  format(1x,a,\)
      write(*,*) ' '
      write(*,'(a\)') ' ERROR :'
      write(*,1111) sgr0, sgr1
      write(*,*) string
      write(*,1111) sgr0
      call sound(15,6000)
      return
      end
