c
c MAGRAV2 ( ver. 2.1) SUBROUTINE BLOCK : MS1 (Halo Professional) 
c Edited last : Oct. 11, 1989; J. Broome
c
$nofloatcalls
c *****************************************************
      FUNCTION DEGSIN(DEGREE)
c
c Purpose : To calculate the sine of degree in degrees
c
      data pi/3.1415926535/
c
      radian = (pi/180.) * degree
      degsin = sin(radian)
      return
      end
c
c *******************************************
      FUNCTION DEGCOS(DEGREE)
c
c Purpose : To calculate the cosine of degree
c  
      data pi/3.1415926535/
c
      radian = (pi/180.) * degree
      degcos = cos(radian)
      return
      end
c
c ***************************************************************
      FUNCTION DELTAG(X1,X,Z1,Z,Y)
c
c  Reference: Thomas Enmark(1981) a versatile interactive
c  computer program for computation and automatic
c  optimization of gravity models; Geoexploration,19,47-66.
c
      xp = x1 - x
      zp = z1 - z
      a0 = sqrt(xp*xp + zp*zp)
      a1 = 1./a0
      fi = atan2(zp,xp)
      zn = xp*a1
      cof = cos(fi)
      sif = sin(fi)
      u = cof*x + sif*z
      u1 = cof*x1 + sif*z1
      w = -sif*x + cof*z
      r = sqrt(u*u + w*w)
      r1 = sqrt(u1*u1 + w*w)
      ak = (x*z1 - z*x1)/(a0*a0)
      rr = sqrt(r*r + y*y)
      rr1 = sqrt(r1*r1 + y*y)
      ratio1 = (y + rr)/(y + rr1)
      rlog1 = alog(r1*ratio1/r)
      ratio2 = y/rr1
      a = ratio2 * u1/w
      ratio3 = y/rr
      b = ratio3 * u/w
      at1 = atan(a)
      at2 = atan(b)
      rlog2 = alog((u1 + rr1)/(u + rr))
      deltag = ak * (zp*rlog1 + xp*(at1-at2)) + zn*rlog2*y
      return
      end
c
c *******************************************************************
      SUBROUTINE DFS001 (M,N,F,X,E,ESCALE,IWRITE,MAXITC,W,IERR)     
c
c This program minimizes the sum of the squares of non linear functions.
c The method used has been developed and described by M.J.D.Powell
c in 'The Computer Journal' Vol.7,No.4,Jan.1965, Page 303.
c The method finds X(1)..........X(N) such that SM is a minimum
c   where SM(X(1)..........X(N)) = SUM 
c   over K of (F(K,X(1)............X(N))**2)
c   where K runs from 1 to M with M  greater than N
c
c The parameter names are as follows
c
c   M =  number of observations of the function F
c   N =  number of independant variables
c
c   F() is array of size greater than M, on leaving DFS001 this will
c    contain the values of F(I,X(1)......X(N))  I = 1 to M
c
c   X() is an array of size greater than n which contains the values of
c    the variables X(1) to X(N). On entering DFS001 they are the initial
c    approximations to the minimum. On leaving DFS001 these will be the
c    best values obtained.
c
c   E() is an array of size equal to 2*N. The first N values are the
c    fractional accuracies of the parameters required.
c    I.E. E(I) = 0.0100 requests an accuracy of 1 in X(I).
c    The rest of the array E(N) to E(2*N) is used as working space.
c    ( Note: E(I) is effectively used as a mesh on the first iteration
c    to form the first derivative of the function F and must therefore
c    be reasonably small escale is a number whose value limits
c    the movement of the variable in any one iteration
c    to an amount equal to ESCALE*E(I))
c
c   IWRITE is an integer which controls the amount of information
c    printed by the routine. there will be a writeout every IWRITE
c    iterations. The writeout consists of the iteration number, number
c    of function evaluations, the values of the variables, the value
c    of the sum of the squares, and the individual function values.
c    If IWRITE is negative, there will be a writeout after every IWRITE
c    iterations but without the function values.
c    If IWRITE is zero, there will be no printed output.
c
c   MAXITC is an integer which will return control to
c    the calling rout after maxitc iterations.
c
c  On leaving DFS001, the first N*N elements of array W() will contain
c  the variance-covariance matrix elements V(I,J) stored in order
c  V(1,1),V(1,2).....V(1,N),V(2,1)............V(N,N).
c
c   W() is a working array whose size must be equal to or greater than
c    N + ((M + (3*N/2))*N + 1))
c
c  The program calls two other routines DFS002 and CALFUN
c  which must calculate the function values F for the passed values
c  of the parameters X(1)......X(N).
c
c  Subroutine DFS002 is supplied intact and finds the minimum of a
c  function in one dimension
c ---------------------------------------------------------------------
c
      dimension f(1), x(1), e(1), w(1)
c
c In this section we initialize some integer constants for the
c  location of information within the array w
c
      do 100 i = 1, n
        if(x(i).eq.0.) then
          call error('Cannot use zero estimate for parameter ')
          write(*,*)' parameter :',i
          ierr = 1
          return
        endif
100   continue
      mplusn = m + n
      kst = n + mplusn
      nplus = n + 1
      kinv = nplus * (mplusn + 1)
      kstore = kinv - mplusn - 1
      nn = n + n
      maxfun = (2*n) + 2 + (6*maxitc)
      invar = 1
c
c The integer invar is normally set to 1 ,after last iteration it
c  is set to 2 and the variance.co-variance matrix is calculated
c
c Stores the fractional accuracy requested and then calculates the
c  absolute values of the errors.
c
      do 110 llm = 1, n
        lln = llm + n
        e(lln) = e(llm)
        e(llm) = e(lln) * x(llm)
110   continue
120   k = nn
c
c This region calculates the first derivatives of the function in
c  the co-ordinate directions and normalizes them such that sum over
c     k of (derivative(i)**2) is unity for all values of i
c
      iamp = 1
      call calfun (iamp)
c
c Stores the initial function values
c
      do 130 i = 1, m
        k = k + 1
        w(k) = f(i)
130   continue
      iinv = 2
      k = kst
      i = 1
140   x(i) = x(i) + e(i)
      call calfun (iamp)
c
c Calculates the function values for  f(x + h) where h = absolute
c     accuracy requested.
c
      x(i) = x(i) - e(i)
      do 150 j = 1, n
        k = k + 1
        w(k) = 0.0
        w(j) = 0.0
150   continue
      sum = 0.0
      kk = nn
c
c Calculates values of the derivatives in the coordinate directions
c     sums the individual derivatives
c
      do 160 j = 1, m
        kk = kk + 1
        f(j) = f(j) - w(kk)
        sum = sum + f(j) * f(j)
160   continue
c
c This error condition usually occurs because of a coding error in
c     the subroutine calfun.
c
      if(sum.le.0) then
        if(iwrite.ne.0) then
          call error(' ')
          write (*,180) i
180       format(' Inversion terminated. Derivative sum',
     +/,' too small to continue, probably due to',/
     +,' roundoff errors.')
        endif
        ierr = i
        do 190 j = 1, m
          nn = nn + 1
          f(j) = w(nn)
190     continue
        return
      endif
c
c  In statement 210, we cancel scaling for calculation of co-variance
c     matrix ,normally we go directly to statement 220
c
200   if(invar.ne.1) then
        sum = e(i) * e(i)
      endif
220   sum = 1.0  / sqrt(sum)
      j = k - n + i
c
c Calculates the components of the direction vectors d(i)
c
      w(j) = e(i) * sum
      do 230 j = 1, m
        k = k + 1
        w(k) = f(j) * sum
        kk = nn + j
c
c Calculates elements of the normal matrix
c
        do 240 ii = 1, i
          kk = kk + mplusn
          w(ii) = w(ii) + w(kk) * w(k)
240     continue
230   continue
c
      iless = i - 1
      igamax = n + i - 1
      incinv = n - iless
      incinp = incinv + 1
c
c Inverts the one by one matrix  w(1)    
c
      if(iless.gt.0) go to 310
        w(kinv) = 1.0/w(1)
260     if(iinv.eq.1) goto 970
        i = i + 1      
        if(i-n.le.0) goto 140
        iinv = 1     
c  
c This region is passed through only on iteration 0 and sets up the  
c       write control parameters     
c  
        ff = 0.0     
        kl = nn      
c  
c Evaluates the sum of the squares of the functions    
c  
        do 270 i = 1, m
          kl = kl + 1
          f(i) = w(kl)
          ff = ff + f(i) * f(i)
270     continue
        if(invar.eq.2) goto 1140
        icont = 1
        iss = 1
        mc = n + 1
        ipp = iabs(iwrite) * (iabs(iwrite)-1)
        itc = 0
        ips = 1
        ipc = 0
        goto 970
c
c This next region performs the matrix inversion of the normal
c   equation matrix by partitioning. the fully inverted matrix is
c   obtained by repeated passes through this region.
c   an outline of the inversion of a matrix by partitioning is given
c   in  elementary matrix algebra  by  f.e.hohn  p.109
c   in  this routine  b = 1/c  and bb = -b/c
c
310   if(invar.eq.1) then
        b = 1.0
      else
        b = w(i)
      endif
      do 320 j = nplus, igamax
        w(j) = 0.0
320   continue
      kk = kinv
      do 330 ii = 1, iless
        iip = ii + n
        w(iip) = w(iip) + w(kk) * w(ii)
        jl = ii + 1
        if(jl-iless.le.0) then
          do 340 jj = jl, iless
            kk = kk + 1
            jjp = jj + n
            w(iip) = w(iip) + w(kk) * w(jj)
            w(jjp) = w(jjp) + w(kk) * w(ii)
340       continue
        endif
        b = b - w(ii) * w(iip)
        kk = kk + incinp
330   continue
      b = 1.0/b
      kk = kinv
c  
c Calculates and stores the elements of the inverted matrix   
c  
      do 350 ii = nplus, igamax
        bb = -b * w(ii)
        do 360 jj = ii, igamax
          w(kk) = w(kk) - bb * w(jj)
          kk = kk + 1
360     continue
        w(kk) = bb
        kk = kk + incinv
350   continue
      w(kk) = b
      goto 260
c
c Now start an iteration, exact details of the method are found in
c     the paper by Powell.
c
420   itc = itc + 1
      if(maxitc-itc.lt.0) then
        if(iwrite.eq.0) goto 1060
        write (*, *) 'The maximum number of allowed iterations ',
     +                  'have been performed'
        goto 1060
      endif
      k = n
      kk = kst
c
c Initially we calculate the vector p
c       where p(i) = sum over k (gamma(k,i)*f(k))
c
      do 430 i = 1, n
        k = k + 1
        w(k) = 0.0
        kk = kk + n
        w(i) = 0.0
        do 440 j = 1, m
          kk = kk + 1
          w(i) = w(i) + w(kk)*f(j)
440     continue
430   continue
c
c Now calculate the movement component q(i) from the inverse deriv-
c   ative matrix and the vector p in the direction d(i)
c
      dm = 0.0
      k = kinv
      do 450 ii = 1, n
        iip = ii + n
        w(iip) = w(iip) + w(k) * w(ii)
        jl = ii + 1
c
c Now select the direction to be replaced by delta after this iter-
c   ation. the direction being replaced is stored in kl
c   the value of abs(p(kl).q(kl))is stored in dm
c
        if(jl-n.le.0) then
          do 460 jj = jl, n
            jjp = jj + n
            k = k + 1
            w(iip) = w(iip) + w(k) * w(jj)
            w(jjp) = w(jjp) + w(k) * w(ii)
460       continue
          k = k + 1
        endif
        if(dm-abs(w(ii)*w(iip)).lt.0.) then
          dm =  abs(w(ii) * w(iip))
          kl = ii
        endif
450   continue
c
c Calculates the absolute error requested
c
      do 465 llm = 1 , n
        lln = llm + n
        e(llm) = e(lln) * x(llm)
465   continue
c
c Calculate the direction and distance to the minimum, a component  
c     is delta(i)  =  q(i)*d(i) where d(i) is an n component vector 
c
      ii = n + mplusn * kl
      change = 0.0
      do 470 i = 1, n
        jl = n + i     
        w(i) = 0.0   
        do 480 j = nplus, nn 
          jl = jl + mplusn      
          w(i) = w(i) + w(j) * w(jl)
480     continue
        ii = ii + 1
        w(ii) = w(jl)
        w(jl) = x(i)
c
c Select change to be max(delta(i)/e(i)) i.e. co-ordinate direction
c   whose distance from the minimum is farthest with respect to the
c   requested accuracy.
c
        if(abs(e(i)*change)-abs(w(i)).le.0.) then
          change =  abs(w(i)/e(i))
        endif
470   continue
c
      do 490 i = 1 , m
        ii = ii + 1
        jl = jl + 1
        w(ii) = w(jl)
        w(jl) = f(i)
490   continue
c
      fc = ff
      acc = 0.10/change
      it = 3
      xc = 0.0
      xl = 0.0
      is = 3
      itmax = 6
      relac = 0.1
      xstep = -amin1(0.5000,escale/change)
c
c Selects a grid value of either 0.500 or escale/change to be a
c   minimum. i.e. in DFS002 we search in one dimension along delta
c   for the minimum value of sm in steps of xstep. therefor if escale
c   is small we will move only small distances in direction of delta-
c   if escale is greater than 0.05 * change, then we move just half
c   way to the computed minimum in one step.
c   The condition of change being less than 1.0d0 is accepted as the
c   minimum. in this next part we enter and return from DFS002 in
c   order to determine the value of lambda such that (su(x(i) + lamda*
c   delta(i)) is a minimum DFS002 assumes that sm approaches the min-
c   imum in the form of a quadratic and calculates on this assumption
c   to perform its task DFS001 asks for a minimum of three evalua-
c   tions of the function. if it has not found the minimum in this
c   number, then the value of xstep is changed. a maximum of just six
c   function evaluations is allowed before going on with the calcula-
c   tion. the function evaluations are stored and used to evaluate
c   the function derivatives in the direction delta.
c
      if(change-1.0.le.0.0) icont = 2
590   call dfs002 (it,xc,fc,itmax,acc,relac,xstep)
c
c  fc - contains intermediate values of the sum of squares
c  xc - contains the value of lambda
c  xl - contains intermediate values of lambda
c 
      goto (600,780,780,780), it
c
600   mc = mc + 1
      if(mc-maxfun.gt.0) then
        if(iwrite.ne.0) then
          call error (' ')
          write(*, *) 'Inversion terminated after ',maxfun
          write(*,*)' iterations (maximum number)'
        endif
        iss = 2
        goto 780
      endif
c
c Calculates the function values for x  + lamda(a)*delta and
c   x + lamda(b)*delta and also evaluates sm for these parameter values
c
630   xl = xc - xl
      do 640 j = 1, n
        x(j) = x(j) + xl * w(j)
640   continue
      xl = xc
      iamp = 0
      call calfun (iamp)
      fc = 0.0
c
c Calculates the new sum of squares of the function.
c
      do 650 j = 1, m
        fc = fc + f(j) * f(j)
650   continue
c
      if(is.le.2) goto 690
      k = n
      if(fc-ff) 670, 590, 680
c
c Sets  fmin to the lowest and fsec to the second lowest sum of
c     squares of the function.
c
670   is = 2
      fmin = fc
      fsec = ff
      goto 750
c
680   is = 1
      fmin = ff
      fsec = fc
      goto 750
c
690   if(fc-fsec.ge.0.0) goto 590
      if(is.ne.2) then
        k = n
      else
        k = kstore
      endif
c
720   if(fc-fmin) 740, 590, 730
730   fsec = fc
      goto 750
740   is = 3 - is
      fsec = fmin
      fmin = fc
c
c Stores intermediate values of the parameters.
c
750   do 760 j = 1, n
        k = k + 1
        w(k) = x(j)
760   continue
c
c Stores the intermediate function values for the lowest and second
c     lowest sums of squares.
c
      do 770 j = 1, m
        k = k + 1
        w(k) = f(j)
770   continue
c
c At this point we return to DFS002
c
      goto 590
780   k = kstore
c
c ===========================================================
c  This is an alternate exit for this application
c
      if(itc.eq.maxitc) return
c
c  We arrive at this point when DFS002 has finished
c
      kk = n
      if(is.eq.0) then
        k = n
        kk = kstore
      endif
800   sum = 0.0
      dm = 0.0
      jj = kstore
c
c We store the new values of x(i), f(i) and the approximate new
c     derivatives.
c
      do 810 j = 1, n
        k = k + 1
        kk = kk + 1
        jj = jj + 1
        x(j) = w(k)
        w(jj) = w(k) - w(kk)
810   continue
c
c Again, calculate a scaling factor,this time for the new derivative
c   calculates dm as sum over k of  u(k,delta)*f(k,new(x(i)))
c
      do 820 j = 1, m
        k = k + 1
        kk = kk + 1
        jj = jj + 1
        f(j) = w(k)
        w(jj) = w(k) - w(kk)
        sum = sum + w(jj) * w(jj)
        dm = dm + f(j) * w(jj)
820   continue
c
      if(iss.eq.2) goto 1060
      j = kinv
      kk = nplus - kl
c
c Repositions  elements in the normal equation matrix and in the     
c     inverted matrix.  
c
      do 830 i = 1, kl
        k = j + kl - i
        j = k + kk
        w(i) = w(k)
        w(k) = w(j-1)
830   continue
c
      if(kl-n.lt.0) then
        kl = kl + 1
        jj = k
        do 840 i = kl, n
          k = k + 1
          j = j + nplus - i
          w(i) = w(k)
          w(k) = w(j-1)
840     continue
        w(jj) = w(k)
        b = 1.0/w(kl-1)
        w(kl-1) = w(n)
      else
        b = 1.0/w(n)
      endif
880   k = kinv
      do 890 i = 1, iless
        bb = b * w(i)
        do 900 j = i, iless
          w(k) = w(k) - bb * w(j)
          k = k + 1
900     continue
        k = k + 1
890   continue
      if(fmin-ff.ge.0.) then
        change = 0.0
      else
        ff = fmin
        change =  abs(xc) * change
      endif
930   xl = -dm/fmin
      dum = sum + dm * xl
      if(dum.le.0.) then
        call error('Inversion terminated')
        call bold('probable cause: roundoff error')
        ierr = 1
        return
      endif
      sum = 1.0/sqrt(dum)
      k = kstore
c
c Calculates the components of the new vector direction.
c
      do 940 i = 1, n
        k = k + 1
        w(k) = sum * w(k)
        w(i) = 0.0
940   continue
c
c Calculates the new and corrected derivatives of the function in
c     the direction of delta
c
      do 950 i = 1, m
        k = k + 1
        w(k) = sum * (w(k) + xl*f(i))
        kk = nn + i
c
c Replaces elements in the matrix i.e. changes the kl direction for
c
        do 960 j = 1, n
          kk = kk + mplusn
          w(j) = w(j) + w(kk) * w(k)
960     continue
950   continue
c
c Go back for new matrix inversion
c
      goto 310
970   ipc = ipc - iabs(iwrite)
c
c The following instruction controls the output of information
c
      if(ipc.ge.0.0) goto 1040
980   if(iwrite.eq.0) goto 1030
      write(*, 990) itc, mc, ff
990   format(//,5x,'iteration',i4,i9,' calls of calfun',5x,'f=',1pe20.
     +8)
      write(*,1000) (x(i),i = 1,n)
1000  format(5x,'parameters',/,(1p5e20.8))
      if(iwrite.ge.0) then
1010    write (*,1020) (f(i),i = 1,m)
1020    format (5x,'Functions',/,(1p5e20.8))
      endif
1030  ipc = ipp
      if(ips.eq.2) goto 1120
1040  if(icont.eq.1) goto 420
      if(change-1..gt.0.0) goto 1130
1060  if(iwrite) 1070, 1120, 1090
1070  write(*, *) 'DFS001 final values of variables'
      goto 1110
1090  write (*, *) 'DFS001 final values of functions and variables'
1110  ips = 2
      goto 980
1120  invar = 2
      goto 120
1130  icont = 1
      goto 420
c
c The statements below store the variance co-variance matrix in the
c     array w
c
1140  jjvar = kinv - 1
      ff = ff/(m-n)
      do 1150 jvar = 1, n
        do 1160 ivar = jvar, n
          jjvar = jjvar + 1
          jkvar = (jvar-1) * n + ivar
          w(jkvar) = w(jjvar) * ff
1160    continue
        if(jvar-1.gt.0) then
          lvar = jvar - 1
          mvar = 0
          lkvar = lvar * n
          do 1170 kvar = 1, lvar
            jkvar = lkvar + kvar
            jnvar = (kvar-1) * n + jvar - mvar + kinv - 1
            w(jkvar) = w(jnvar) * ff
            mvar = mvar + kvar
1170    continue
        endif
1150  continue
      jvar = n * n
      return
      end
c
*****************************************************************
      SUBROUTINE DFS002 (ITEST,X,F,MAXFUN,ABSACC,RELACC,XSTEP)
c
c This subroutine finds the minimum of a function in one
c  dimension. The method used has been described by M.J.D. Powell in
c  'The Computer Journal',vol.7,number 2, July 1964,p.155.
c  The method assumes that the function approaches the minimum
c  quadratically.  On first entry it has a value of the function at
c  one point. It then calculates two additional points in the direc-
c  tion requested (returning to the calling routine to do so). the
c  method then predicts the minimum of the  quadratic that passes
c  through the three data points.  If the minimum is bracketed by the
c  three data points then minimum predicted is used.  If it is not
c  then further steps are taken along the direction requested. a
c  total number of steps maxfun is allowed.
c
c  itest - is a control integer which must be set to 2 or 3 on initial
c     entry into the routine and to 1 on subsequent entries during the
c     same search.
c
c  x  - contains the distance being moved on intermediate returns to
c     the calling routine and the distance to the minimum on the final
c     return f is used to transmit the function values.
c
c  maxfun - is the total number of function evaluations allowed , if
c     maxfun is exceeded then routine returns with the nearest value to
c     the minimum in x
c
c  absacc - is the absolute accuracy required for the minimum
c
c  relacc - is the relative accuracy required for the minimum
c
c  xstep - is the stepping or increment distance in the direction of
c     the minimum
c
      if(itest.eq.1) goto 7
c
c For the first entry into DFS002, itest is set equal to 2 or 3 which
c   sets up the initial conditions for the computation. itest = 3
c   stores the initial function value and increments x. itest = 2
c   causes a return to the calling routine requesting a first value
c   of the function.
c
c After setting the intial conditions, itest is reset to 1 until the
c   minimum has been obtained. Itest  = 2 is a sucessful calculation
c   itest  =  3 is also a sucessful completion ,while itest  =  4 is a
c   return that indicates that maxfun has been exceeded.
c
      is = 6 - itest
      itest = 1
      iinc = 1
      xinc = xstep + xstep
      mc = is - 3
      if(mc) 9, 9, 6
c
c Increments function call counter ,if less than maxfun return to
c   calling routine for new function evaluation
c
2     mc = mc + 1
      if(maxfun-mc.ge.0) return
3     itest = 4
4     x = db
      f = fb
      if(fb-fc.gt.0.0) then
        x = dc
        f = f
      endif
6     return
7     goto (17,15,10,8), is
8     is = 3
c
c We come to this point after calculating the first function
c   value for the itest = 2 initial option ,from the next statement (4)
c   on itest  = 3 and itest  = 2 are identical.
c   Stores intial function and position values
c
9     dc = x
      fc = f
c
c Increments position
c
      x = x + xstep
      goto 2
10    if(fc-f) 12, 11, 13
c
c Comes to this point after evaluating second function, goes to 10
c   if first  + second function values are identical, goes to 11 if new
c   one is less and to 9 if new one is larger
c
11    x = x + xinc
c
c In this section, nothing is stored as new and old function values
c   are identical , but x is increment and a new function values is
c   requested , is remains unchanged so we come back to label 7 again
c
      xinc = xinc + xinc
      goto 2
12    db = x
c
c In this section, the new function value is larger than the initial
c   one ,again larger one is put into fb , it then changes the sign of
c   xinc and then increments again along x the same amount as for
c   Section 11.
c
      fb = f
      xinc = -xinc
      goto 14
13    db = dc
c
c In this section, the new function value is less than the initial
c   one. it stores the higher in fb and the lower in fc. x is incre-
c   mented again in the same direction
c   IS is changed to 2 so next entry brings calculation to label 6
c
      fb = fc
      dc = x
      fc = f
14    x = dc + dc - db
      is = 2
      goto 2
15    da = db
c
c Arrives in this section having obtained the third function value
c   the largest of the first two is stored in fa and the smallest in
c   fb with the third in fc
c
      db = dc
      fa = fb
      fb = fc
16    dc = x
      fc = f
      goto 27
c
c Comes to this section when minimum was within the three function
c   values but not close enough to one end. all the next does is
c   decide which function values to take and in what order so as to
c   minimize rounding errors
c
17    if(fb-fc.lt.0) go to 21
        if(f-fb.ge.0.0) goto 16
19      fa = fb
        da = db
20      fb = f
        db = x
        goto 27
21    if(fa-fc.gt.0.0) then
        xinc = fa
        fa = fc
        fc = xinc
        xinc = da
        da = dc
        dc = xinc
      endif
23    xinc = dc
      if((d-db)*(d-dc).lt.0.0) goto 16
24    if(f-fa.lt.0.0) then
        fc = fb
        dc = db
        goto 20
      endif
26    fa = f
      da = x
c
c Tests to see if the third is smallest. if so, goto 29 and cal-
c   culate second derivative
c
c If third function is equal to or larger than the smallest,come to
c   this point and set xinc to twice xstep and sets integer iinc to 2,
c   this integer controls later computations . still calculates the
c   second derivative ,provided that the function values fb and fc are
c   not equal. if they are we goto label 45 where we recalculate with
c   a new x value equal to the mid point of fb and fc which gives a
c   new function value
c
27    if(fb-fc.le.0.0) then
        iinc = 2     
        xinc = dc    
        if(fb-fc.eq.0.) goto 40
      endif
29    if((da-db).eq.0) goto 3
      if((da-dc).eq.0) goto 3
      d = (fa-fb)/(da-db) - (fa-fc)/(da-dc)
c
c Tests sign of second derivative if negative there is a minimum
c   at the calculated value of d , if positive we have found a maximum
c   and goto label 33 to restore calculation for a minimum
c
c Having calculated a minimum, now test for it occuring near enough
c   to the last function evaluation . if it is within either the
c   absolute or relative accuracy then terminate with itest  =  2
c   if not we goto label 36.
c
      if(d*(db-dc).le.0.0) goto 36
30    d = 0.50 * (db + dc-(fb-fc)/d)
      if(abs(d-x)-abs(absacc).le.0.0.or.
     +   abs(d-x)-abs(d*relacc).le.0.0) then
        itest = 2
        goto 4
      endif
33    is = 1
c
c Set x to predicted value of minimum ,if this is within the range
c   examined previously go back with is = 1 and calculate new function
c   value. if outside previous range we recalculate the third function
c   value. if less then 4*increment use predicted value for x. if
c   greater then use 4*increment
c
      x = d
      if((da-dc)*(dc-d)) 2, 41, 34
34    is = 2
      if(iinc.eq.2) goto 38
      if(abs(xinc)-abs(dc-d)) 37, 2, 2
36    is = 2
      if(iinc.eq.2) goto 39
37    x = dc
      goto 11
38    if(abs(xinc-x)-abs(x-dc).gt.0.0) goto 2
39    x = 0.50 * (xinc + dc)
      if((xinc-x)*(x-dc)) 41, 41, 2
40    x = 0.50 * (db + dc)
      if((db-x)*(x-dc).gt.0.0) goto 2
41    itest = 3
      goto 4
      end
